@@ -1,6 +1,6 @@
-'''
+"""
 Definition for an RPC channel protocol on top of a websocket - enabling bi-directional request/response interactions
-'''
+"""
 import asyncio
 from inspect import _empty, getmembers, ismethod, signature
 from typing import Any, Coroutine, Dict, List
@@ -9,7 +9,7 @@ from asyncio.coroutines import coroutine
 from pydantic import ValidationError
 
 from .utils import gen_uid
-from .rpc_methods import NoResponse, RpcMethodsBase
+from .rpc_methods import EXPOSED_BUILT_IN_METHODS, NoResponse, RpcMethodsBase
 from .schemas import RpcMessage, RpcRequest, RpcResponse
 
 from .logger import get_logger
@@ -19,6 +19,8 @@ logger = get_logger("RPC_CHANNEL")
 class DEAFULT_TIMEOUT:
     pass
 
+class RemoteValueError(ValueError):
+    pass
 
 class RpcException(Exception):
     pass
@@ -55,13 +57,13 @@ class RpcPromise:
 
     def set(self):
         """
-        Signal compeltion of request with received response 
+        Signal compeltion of request with received response
         """
         self._event.set()
 
     def wait(self):
         """
-        Wait on the internal event - triggered on response  
+        Wait on the internal event - triggered on response
         """
         return self._event.wait()
 
@@ -90,14 +92,15 @@ class RpcCaller:
         self._method_names = [method[0] for method in getmembers(
             methods, lambda i: ismethod(i))] if methods is not None else None
 
+
     def __getattribute__(self, name: str):
-        if (not name.startswith("_") or name == "_ping_") and (self._method_names is None or name in self._method_names):
+        if (not name.startswith("_") or name in EXPOSED_BUILT_IN_METHODS) and (self._method_names is None or name in self._method_names):
             return RpcProxy(self._channel, name)
         else:
             return super().__getattribute__(name)
 
 
-# Callback signatures 
+# Callback signatures
 async def OnConnectCallback(channel):
     pass
 
@@ -117,7 +120,7 @@ class RpcChannel:
     e.g. answer = channel.other.add(a=1,b=1) will (For example) ask the other side to perform 1+1 and will return an RPC-response of 2
     """
 
-    def __init__(self, methods: RpcMethodsBase, socket, channel_id=None, default_response_timeout=None, **kwargs):
+    def __init__(self, methods: RpcMethodsBase, socket, channel_id=None, default_response_timeout=None, sync_channel_id=False, **kwargs):
         """
 
         Args:
@@ -125,6 +128,8 @@ class RpcChannel:
             socket: socket object providing simple send/recv methods
             channel_id (str, optional): uuid for channel. Defaults to None in which case a random UUID is generated.
             default_response_timeout(int, optional) default timeout for RPC call responses. Defaults to None - i.e. no timeout
+            sync_channel_id(bool, optional) should get the other side of the channel id, helps to identify connections, cost a bit networking time.
+                Defaults to False - i.e. not getting the other side channel id
         """
         self.methods = methods._copy_()
         # allow methods to access channel (for recursive calls - e.g. call me as a response for me calling you)
@@ -138,6 +143,12 @@ class RpcChannel:
         self.default_response_timeout = default_response_timeout
         # Unique channel id
         self.id = channel_id if channel_id is not None else gen_uid()
+        # flag control should we retrieve the channel id of the other side
+        self._sync_channel_id = sync_channel_id
+        # The channel id of the other side (if sync_channel_id is True)
+        self._other_channel_id = None
+        # asyncio event to check if we got the channel id of the other side
+        self._channel_id_synced = asyncio.Event()
         #
         # convineice caller
         # TODO - pass remote methods object to support validation before call
@@ -156,6 +167,15 @@ class RpcChannel:
     def context(self) -> Dict[str, Any]:
         return self._context
 
+    async def get_other_channel_id(self) -> str:
+        """
+        Method to get the channel id of the other side of the channel
+        The _channel_id_synced verify we have it
+        Timeout exception can be raised if the value isn't available
+        """
+        await asyncio.wait_for(self._channel_id_synced.wait(), self.default_response_timeout)
+        return self._other_channel_id
+
     def get_return_type(self, method):
         method_signature = signature(method)
         return method_signature.return_annotation if method_signature.return_annotation is not _empty else str
@@ -180,7 +200,7 @@ class RpcChannel:
 
     def isClosed(self):
         return self._closed.is_set()
-    
+
     async def wait_until_closed(self):
         return await self._closed.wait()
 
@@ -208,22 +228,22 @@ class RpcChannel:
         Register a connection handler callback that will be called (As an async task)) with the channel
         Args:
             coros (List[Coroutine]): async callback
-        """        
+        """
         if coros is not None:
             self._connect_handlers.extend(coros)
 
     def register_disconnect_handler(self, coros:List[OnDisconnectCallback]=None):
         """
-        Register a disconnect handler callback that will be called (As an async task)) with the channel id 
+        Register a disconnect handler callback that will be called (As an async task)) with the channel id
         Args:
             coros (List[Coroutine]): async callback
         """
-        if coros is not None:        
+        if coros is not None:
             self._disconnect_handlers.extend(coros)
 
     def register_error_handler(self, coros:List[OnErrorCallback]=None):
         """
-        Register an error handler callback that will be called (As an async task)) with the channel and triggered error. 
+        Register an error handler callback that will be called (As an async task)) with the channel and triggered error.
         Args:
             coros (List[Coroutine]): async callback
         """
@@ -234,8 +254,30 @@ class RpcChannel:
         await asyncio.gather(*(callback(*args, **kwargs) for callback in handlers))
 
     async def on_connect(self):
+        """
+        Run get other channel id if sync_channel_id is True
+        Run all callbacks from self._connect_handlers
+        """
+        if self._sync_channel_id:
+            self._get_other_channel_id_task = asyncio.create_task(self._get_other_channel_id())
         await self.on_handler_event(self._connect_handlers, self)
 
+    async def _get_other_channel_id(self):
+        """
+        Perform call to the other side of the channel to get its channel id
+        Each side is generating the channel id by itself so there is no way to identify a connection without this sync
+        """
+        if self._other_channel_id is None:
+            other_channel_id = await self.other._get_channel_id_()
+            self._other_channel_id = other_channel_id.result if other_channel_id and other_channel_id.result else None
+            if self._other_channel_id is None:
+                raise RemoteValueError()
+            # update asyncio event that we received remote channel id
+            self._channel_id_synced.set()
+            return self._other_channel_id
+        else:
+            return self._other_channel_id
+
     async def on_disconnect(self):
         # disconnect happend - mark the channel as closed
         self._closed.set()
@@ -247,7 +289,7 @@ class RpcChannel:
     async def on_request(self, message: RpcRequest):
         """
         Handle incoming RPC requests - calling relevant exposed method
-        Note: methods prefixed with "_" are protected and ignored. 
+        Note: methods prefixed with "_" are protected and ignored.
 
         Args:
             message (RpcRequest): the RPC request with the method to call
@@ -256,7 +298,7 @@ class RpcChannel:
         logger.debug("Handling RPC request - %s", {'request':message, 'channel':self.id})
         method_name = message.method
         # Ignore "_" prefixed methods (except the built in "_ping_")
-        if (isinstance(method_name, str) and (not method_name.startswith("_") or method_name == "_ping_")):
+        if (isinstance(method_name, str) and (not method_name.startswith("_") or method_name in EXPOSED_BUILT_IN_METHODS)):
             method = getattr(self.methods, method_name)
             if callable(method):
                 result = await method(**message.arguments)
